@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 1999-2018 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 1999-2019 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -12,6 +12,8 @@
 #ifndef _NV_H_
 #define _NV_H_
 
+
+
 #if !defined(NVLIMITS_H_MISSING)
 #include <nvlimits.h>
 #else
@@ -291,7 +293,6 @@ typedef union
 typedef struct
 {
     NvU64 cpu_address;
-    NvU64 bus_address;
     NvU64 strapped_size;
     NvU64 size;
     NvU32 offset;
@@ -442,14 +443,20 @@ typedef struct
     NvU64 dma_addressable_limit;
 
     NvBool nvlink_sysmem_links_enabled;
+
+    /*
+     * Per-GPU queue.  The actual queue object is usually allocated in the
+     * arch-specific parent structure (e.g. nv_linux_state_t), and this
+     * pointer just points to it.
+     */
+    struct os_work_queue *queue;
 } nv_state_t;
 
 // Forward define the gpu ops structures
 typedef struct gpuSession                           *nvgpuSessionHandle_t;
 typedef struct gpuAddressSpace                      *nvgpuAddressSpaceHandle_t;
 typedef struct gpuChannel                           *nvgpuChannelHandle_t;
-typedef struct gpuObject                            *nvgpuObjectHandle_t;
-typedef struct UvmGpuChannelPointers_tag            *nvgpuChannelInfo_t;
+typedef struct UvmGpuChannelInfo_tag                *nvgpuChannelInfo_t;
 typedef struct UvmGpuChannelAllocParams_tag          nvgpuChannelAllocParams_t;
 typedef struct UvmGpuCaps_tag                       *nvgpuCaps_t;
 typedef struct UvmGpuCopyEnginesCaps_tag            *nvgpuCesCaps_t;
@@ -484,19 +491,45 @@ typedef NV_STATUS (*nvPmaEvictRangeCallback)(void *, NvU64, NvU64);
 #define NV_FLAG_USES_MSI               0x0020
 #define NV_FLAG_USES_MSIX              0x0040
 #define NV_FLAG_PASSTHRU               0x0080
-#define NV_FLAG_GVI_IN_SUSPEND         0x0100
+#define NV_FLAG_SUSPENDED              0x0100
 #define NV_FLAG_GVI                    0x0200
 #define NV_FLAG_GVI_INTR_EN            0x0400
 #define NV_FLAG_PERSISTENT_SW_STATE    0x0800
 #define NV_FLAG_IN_RECOVERY            0x1000
-#define NV_FLAG_SKIP_CFG_CHECK         0x2000
+// Unused                              0x2000
 #define NV_FLAG_UNBIND_LOCK            0x4000
 /* To be set when GPU is not present on the bus, to help device teardown */
 #define NV_FLAG_IN_SURPRISE_REMOVAL    0x8000
 
-#define NV_PM_ACPI_HIBERNATE    0x0001
-#define NV_PM_ACPI_STANDBY      0x0002
-#define NV_PM_ACPI_RESUME       0x0003
+typedef enum
+{
+    NV_PM_ACTION_HIBERNATE,
+    NV_PM_ACTION_STANDBY,
+    NV_PM_ACTION_RESUME
+} nv_pm_action_t;
+
+typedef enum
+{
+    NV_PM_ACTION_DEPTH_DEFAULT,
+    NV_PM_ACTION_DEPTH_MODESET,
+    NV_PM_ACTION_DEPTH_UVM
+} nv_pm_action_depth_t;
+
+typedef enum
+{
+    NV_DYNAMIC_PM_NEVER,
+    NV_DYNAMIC_PM_COARSE,
+
+
+
+} nv_dynamic_power_mode_t;
+
+typedef enum
+{
+    NV_POWER_STATE_IN_HIBERNATE,
+    NV_POWER_STATE_IN_STANDBY,
+    NV_POWER_STATE_RUNNING
+} nv_power_state_t;
 
 #define NV_PRIMARY_VGA(nv)      ((nv)->primary_vga)
 
@@ -639,8 +672,6 @@ typedef NV_STATUS (*nvPmaEvictRangeCallback)(void *, NvU64, NvU64);
  * driver internal interfaces
  */
 
-#ifndef NVWATCH
-
 /*
  * ---------------------------------------------------------------------------
  *
@@ -693,8 +724,6 @@ NvS32  NV_API_CALL  nv_stop_rc_timer             (nv_state_t *);
 void   NV_API_CALL  nv_post_event                (nv_state_t *, nv_event_t *, NvHandle, NvU32, NvBool);
 NvS32  NV_API_CALL  nv_get_event                 (nv_state_t *, void *, nv_event_t *, NvU32 *);
 
-void   NV_API_CALL  nv_verify_pci_config         (nv_state_t *, BOOL);
-
 void*  NV_API_CALL  nv_i2c_add_adapter           (nv_state_t *, NvU32);
 BOOL   NV_API_CALL  nv_i2c_del_adapter           (nv_state_t *, void *);
 
@@ -713,10 +742,6 @@ NV_STATUS  NV_API_CALL  nv_set_primary_vga_status(nv_state_t *);
 NV_STATUS  NV_API_CALL  nv_pci_trigger_recovery  (nv_state_t *);
 NvBool     NV_API_CALL  nv_requires_dma_remap    (nv_state_t *);
 
-#if defined(NVCPU_X86) || defined(NVCPU_X86_64)
-NvBool     NV_API_CALL  nv_is_virtualized_system  (nv_stack_t *);
-#endif
-
 NV_STATUS NV_API_CALL nv_export_rm_object_to_fd    (NvHandle, NvS32);
 NV_STATUS NV_API_CALL nv_import_rm_object_from_fd  (NvS32, NvHandle *);
 
@@ -737,6 +762,16 @@ NV_STATUS NV_API_CALL nv_get_nvlink_line_rate    (nv_state_t *, NvU32 *);
 
 void      NV_API_CALL nv_register_backlight      (nv_state_t *, NvU32, NvU32);
 void      NV_API_CALL nv_unregister_backlight    (nv_state_t *);
+
+NV_STATUS NV_API_CALL nv_revoke_gpu_mappings     (nv_state_t *);
+void      NV_API_CALL nv_acquire_mmap_lock       (nv_state_t *);
+void      NV_API_CALL nv_release_mmap_lock       (nv_state_t *);
+NvBool    NV_API_CALL nv_get_all_mappings_revoked_locked (nv_state_t *);
+void      NV_API_CALL nv_set_safe_to_mmap_locked (nv_state_t *, NvBool);
+
+NV_STATUS NV_API_CALL nv_indicate_idle           (nv_state_t *);
+NV_STATUS NV_API_CALL nv_indicate_not_idle       (nv_state_t *);
+
 /*
  * ---------------------------------------------------------------------------
  *
@@ -760,7 +795,9 @@ NV_STATUS  NV_API_CALL  rm_ioctl                 (nvidia_stack_t *, nv_state_t *
 BOOL       NV_API_CALL  rm_isr                   (nvidia_stack_t *, nv_state_t *, NvU32 *);
 void       NV_API_CALL  rm_isr_bh                (nvidia_stack_t *, nv_state_t *);
 void       NV_API_CALL  rm_isr_bh_unlocked       (nvidia_stack_t *, nv_state_t *);
-NV_STATUS  NV_API_CALL  rm_power_management      (nvidia_stack_t *, nv_state_t *, NvU32, NvU32);
+NV_STATUS  NV_API_CALL  rm_power_management      (nvidia_stack_t *, nv_state_t *, nv_pm_action_t);
+NV_STATUS  NV_API_CALL  rm_stop_user_channels    (nvidia_stack_t *, nv_state_t *);
+NV_STATUS  NV_API_CALL  rm_restart_user_channels (nvidia_stack_t *, nv_state_t *);
 NV_STATUS  NV_API_CALL  rm_save_low_res_mode     (nvidia_stack_t *, nv_state_t *);
 NV_STATUS  NV_API_CALL  rm_get_vbios_version     (nvidia_stack_t *, nv_state_t *, NvU32 *, NvU32 *, NvU32 *, NvU32 *, NvU32 *);
 NV_STATUS  NV_API_CALL  rm_get_gpu_uuid          (nvidia_stack_t *, nv_state_t *, NvU8 **, NvU32 *);
@@ -827,9 +864,8 @@ NV_STATUS  NV_API_CALL  rm_gpu_ops_get_p2p_caps(nvidia_stack_t *, nvgpuAddressSp
 
 NV_STATUS  NV_API_CALL  rm_gpu_ops_memory_cpu_map(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvU64, NvLength, void **, NvU32);
 NV_STATUS  NV_API_CALL  rm_gpu_ops_memory_cpu_ummap(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, void*);
-NV_STATUS  NV_API_CALL  rm_gpu_ops_channel_allocate(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, nvgpuChannelHandle_t *, const nvgpuChannelAllocParams_t *, nvgpuChannelInfo_t);
+NV_STATUS  NV_API_CALL  rm_gpu_ops_channel_allocate(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, const nvgpuChannelAllocParams_t *, nvgpuChannelHandle_t *, nvgpuChannelInfo_t);
 NV_STATUS  NV_API_CALL  rm_gpu_ops_channel_destroy(nvidia_stack_t *, nvgpuChannelHandle_t);
-NV_STATUS  NV_API_CALL  rm_gpu_ops_copy_engine_alloc(nvidia_stack_t *, nvgpuChannelHandle_t, NvU32, nvgpuObjectHandle_t *, nvgpuChannelInfo_t);
 NV_STATUS  NV_API_CALL rm_gpu_ops_memory_free(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, NvU64);
 NV_STATUS  NV_API_CALL rm_gpu_ops_query_caps(nvidia_stack_t *, nvgpuAddressSpaceHandle_t, nvgpuCaps_t);
 NV_STATUS  NV_API_CALL rm_gpu_ops_query_ces_caps(nvidia_stack_t *sp, nvgpuAddressSpaceHandle_t, nvgpuCesCaps_t);
@@ -880,6 +916,15 @@ NV_STATUS  NV_API_CALL rm_gpu_numa_offline(nvidia_stack_t *, nv_state_t *);
 NvBool     NV_API_CALL rm_is_device_sequestered(nvidia_stack_t *, nv_state_t *);
 void       NV_API_CALL rm_check_for_gpu_surprise_removal(nvidia_stack_t *, nv_state_t *);
 NV_STATUS  NV_API_CALL rm_set_external_kernel_client_count(nvidia_stack_t *, nv_state_t *, NvBool);
+NV_STATUS  NV_API_CALL rm_schedule_gpu_wakeup(nvidia_stack_t *, nv_state_t *);
+NV_STATUS  NV_API_CALL rm_gpu_gc6_blocker_refcount_inc(nvidia_stack_t *, nv_state_t *);
+NV_STATUS  NV_API_CALL rm_gpu_gc6_blocker_refcount_dec(nvidia_stack_t *, nv_state_t *);
+
+NV_STATUS  NV_API_CALL rm_init_dynamic_power_management(nvidia_stack_t *, nv_state_t *);
+void       NV_API_CALL rm_cleanup_dynamic_power_management(nvidia_stack_t *, nv_state_t *);
+NV_STATUS  NV_API_CALL rm_ref_dynamic_power(nvidia_stack_t *, nv_state_t *, nv_dynamic_power_mode_t);
+void       NV_API_CALL rm_unref_dynamic_power(nvidia_stack_t *, nv_state_t *, nv_dynamic_power_mode_t);
+
 /* vGPU VFIO specific functions */
 NV_STATUS  NV_API_CALL  nv_vgpu_create_request(nvidia_stack_t *, nv_state_t *, NvU8 *, NvU32, NvU16 *, NvU32);
 NV_STATUS  NV_API_CALL  nv_vgpu_delete(nvidia_stack_t *, NvU8 *, NvU16);
@@ -888,11 +933,10 @@ NV_STATUS  NV_API_CALL  nv_vgpu_get_type_info(nvidia_stack_t *, nv_state_t *, Nv
 NV_STATUS  NV_API_CALL  nv_vgpu_get_bar_info(nvidia_stack_t *, nv_state_t *, NvU8 *, NvU64 *, NvU32, void *, void **);
 NV_STATUS  NV_API_CALL  nv_vgpu_start(nvidia_stack_t *, NvU8 *, void *, NvS32 *, NvU8 *, NvU32, NvU8 *);
 NV_STATUS  NV_API_CALL  nv_vgpu_get_sparse_mmap(nvidia_stack_t *, nv_state_t *, NvU8 *, NvU64 **, NvU64 **, NvU32 *);
-NV_STATUS  NV_API_CALL  nv_vgpu_update_request(nvidia_stack_t *, NvU8 *, NvU32);
+NV_STATUS  NV_API_CALL  nv_vgpu_update_request(nvidia_stack_t *, NvU8 *, NvU32, NvU64 *, NvU64 *);
 NV_STATUS  NV_API_CALL  nv_gpu_bind_event(nvidia_stack_t *);
 
 NV_STATUS NV_API_CALL nv_get_usermap_access_params(nv_state_t*, nv_usermap_access_params_t*);
-#endif /* NVWATCH */
 
 static inline const NvU8 *nv_get_cached_uuid(nv_state_t *nv)
 {
