@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016-2018 NVIDIA Corporation
+    Copyright (c) 2016-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -23,10 +23,10 @@
 
 #include <linux/version.h>
 
-#include "nv-procfs.h"
 #include "conftest.h"
 #include "nvmisc.h"
 #include "nv-linux.h"
+#include "nv-procfs.h"
 #include "nvlink_common.h"
 #include "nvlink_errors.h"
 #include "nv-kthread-q.h"
@@ -131,6 +131,7 @@ static struct pci_driver nvswitch_pci_driver =
     .id_table       = nvswitch_pci_table,
     .probe          = nvswitch_probe,
     .remove         = nvswitch_remove,
+    .shutdown       = nvswitch_remove
 };
 
 //
@@ -203,6 +204,7 @@ typedef struct
     struct pci_dev *pci_dev;
     atomic_t ref_count;
     struct list_head list_node;
+    NvBool unusable;
 } NVSWITCH_DEV;
 
 // Global driver state
@@ -268,6 +270,13 @@ static void nvswitch_shutdown_device_interrupt(NVSWITCH_DEV *nvswitch_dev);
 static void nvswitch_load_bar_info(NVSWITCH_DEV *nvswitch_dev);
 static void nvswitch_task_dispatch(NVSWITCH_DEV *nvswitch_dev);
 
+#if defined(CONFIG_PROC_FS)
+#define NV_DEFINE_SINGLE_NVSWITCH_PROCFS_FILE(name) \
+    NV_DEFINE_SINGLE_PROCFS_FILE(name, \
+                                 NV_READ_LOCK_SYSTEM_PM_LOCK_INTERRUPTIBLE, \
+                                 NV_READ_UNLOCK_SYSTEM_PM_LOCK)
+#endif
+
 #define NVSWITCH_PROCFS_DIR "driver/nvidia-nvswitch"
 
 static struct proc_dir_entry *nvswitch_procfs_dir = NULL;
@@ -293,7 +302,7 @@ nv_procfs_read_permissions
     return 0;
 }
 
-NV_DEFINE_PROCFS_SINGLE_FILE(permissions);
+NV_DEFINE_SINGLE_NVSWITCH_PROCFS_FILE(permissions);
 
 static void
 nvswitch_permissions_exit
@@ -628,7 +637,15 @@ nvswitch_device_release
 
     file->private_data = NULL;
 
-    NV_ATOMIC_DEC(nvswitch_dev->ref_count);
+    //
+    // If there are no outstanding references and the device is marked
+    // unusable, free it.
+    //
+    if (NV_ATOMIC_DEC_AND_TEST(nvswitch_dev->ref_count) &&
+        nvswitch_dev->unusable)
+    {
+        kfree(nvswitch_dev);
+    }
 
     mutex_unlock(&nvswitch.driver_mutex);
 
@@ -653,6 +670,13 @@ nvswitch_device_poll
         return rc;
     }
 
+    if (nvswitch_dev->unusable)
+    {
+        printk(KERN_INFO "%s: a stale fd detected\n", nvswitch_dev->name);
+        rc = POLLHUP;
+        goto done;
+    }
+
     poll_wait(file, &nvswitch_dev->wait_q_errors, wait);
 
     // Allow user to poll on either fatal, non-fatal or both types of errors.
@@ -660,6 +684,7 @@ nvswitch_device_poll
     rc = fatal ? POLLPRI : 0;
     rc |= nonfatal ? POLLIN : 0;
 
+done:
     mutex_unlock(&nvswitch_dev->device_mutex);
 
     return rc;
@@ -788,6 +813,13 @@ nvswitch_device_ioctl
         return rc;
     }
 
+    if (nvswitch_dev->unusable)
+    {
+        printk(KERN_INFO "%s: a stale fd detected\n", nvswitch_dev->name);
+        rc = -ENODEV;
+        goto nvswitch_device_ioctl_exit;
+    }
+
     rc = nvswitch_ioctl_state_start(&state, cmd, arg);
     if (rc)
     {
@@ -973,9 +1005,6 @@ nvswitch_isr_pending
 (
     int   irq,
     void *arg
-#if !defined(NV_IRQ_HANDLER_T_PRESENT) || (NV_IRQ_HANDLER_T_ARGUMENT_COUNT == 3)
-    ,struct pt_regs *regs
-#endif
 )
 {
     NVSWITCH_DEV *nvswitch_dev = (NVSWITCH_DEV *)arg;
@@ -1025,9 +1054,6 @@ nvswitch_isr_thread
 (
     int   irq,
     void *arg
-#if !defined(NV_IRQ_HANDLER_T_PRESENT) || (NV_IRQ_HANDLER_T_ARGUMENT_COUNT == 3)
-    ,struct pt_regs *regs
-#endif
 )
 {
     NVSWITCH_DEV *nvswitch_dev = (NVSWITCH_DEV *)arg;
@@ -1051,15 +1077,10 @@ nvswitch_isr_thread
 
     mutex_unlock(&nvswitch_dev->device_mutex);
 
-    // PCI errors and handled else where.
-    if (-NVL_PCI_ERROR == retval)
+    if (WARN_ON(retval != NVL_SUCCESS))
     {
-        return IRQ_NONE;
-    }
-
-    if (NVL_SUCCESS != retval)
-    {
-        panic("%s: unrecoverable error in interrupt thread\n", nvswitch_dev->name);
+        printk(KERN_ERR "%s: Interrupts disabled to avoid a storm\n",
+               nvswitch_dev->name);
     }
 
     return IRQ_HANDLED;
@@ -1251,10 +1272,17 @@ nvswitch_remove
     struct pci_dev *pci_dev
 )
 {
-    NVSWITCH_DEV *nvswitch_dev = pci_get_drvdata(pci_dev);
+    NVSWITCH_DEV *nvswitch_dev;
 
     mutex_lock(&nvswitch.driver_mutex);
 
+    nvswitch_dev = pci_get_drvdata(pci_dev);
+
+    if (nvswitch_dev == NULL)
+    {
+        goto done;
+    }
+
     printk(KERN_INFO "%s: removing device %04x:%02x:%02x.%x\n",
            nvswitch_dev->name,
            NV_PCI_DOMAIN_NUMBER(pci_dev),
@@ -1262,24 +1290,13 @@ nvswitch_remove
            NV_PCI_SLOT_NUMBER(pci_dev),
            PCI_FUNC(pci_dev->devfn));
 
-    // Sanity check: A removed device shouldn't have a non-zero ref_count.
-    if (NV_ATOMIC_READ(nvswitch_dev->ref_count) != 0)
-    {
-        printk(KERN_ERR,
-               "%s: attempting to remove device with non-zero ref-count!\n",
-               nvswitch_dev->name);
-
-        WARN_ON(1);
-
-        //
-        // We can't continue without corrupting state, so just hang to give the
-        // user some chance to do something about this before reboot.
-        //
-        while (1)
-        {
-            os_schedule();
-        }
-    }
+    //
+    // Synchronize with device operations such as .ioctls/.poll, and then mark
+    // the device unusable.
+    //
+    mutex_lock(&nvswitch_dev->device_mutex);
+    nvswitch_dev->unusable = NV_TRUE;
+    mutex_unlock(&nvswitch_dev->device_mutex);
 
     NV_ATOMIC_DEC(nvswitch.count);
 
@@ -1297,8 +1314,13 @@ nvswitch_remove
 
     pci_disable_device(pci_dev);
 
-    kfree(nvswitch_dev);
+    // Free nvswitch_dev only if it is not in use.
+    if (NV_ATOMIC_READ(nvswitch_dev->ref_count) == 0)
+    {
+        kfree(nvswitch_dev);
+    }
 
+done:
     mutex_unlock(&nvswitch.driver_mutex);
 
     return;
