@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 2013-2015 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 2013-2019 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -129,6 +129,14 @@ NV_STATUS nvUvmInterfaceRegisterGpu(const NvProcessorUuid *gpuUuid, UvmGpuPlatfo
                                      &gpuInfo->pci_dev,
                                      &gpuInfo->dma_addressable_start,
                                      &gpuInfo->dma_addressable_limit);
+
+        // Block GPU from entering GC6 while used by UVM.
+        if (rc == 0)
+            rc = nvidia_dev_block_gc6(gpuUuid->uuid, sp);
+
+        // Avoid leaking reference on GPU if we failed.
+        if (rc != 0)
+            nvidia_dev_put_uuid(gpuUuid->uuid, sp);
     }
 
     switch (rc)
@@ -155,6 +163,7 @@ EXPORT_SYMBOL(nvUvmInterfaceRegisterGpu);
 void nvUvmInterfaceUnregisterGpu(const NvProcessorUuid *gpuUuid)
 {
     nvidia_stack_t *sp = nvUvmGetSafeStack();
+    nvidia_dev_unblock_gc6(gpuUuid->uuid, sp);
     nvidia_dev_put_uuid(gpuUuid->uuid, sp);
     nvUvmFreeSafeStack(sp);
 }
@@ -500,9 +509,9 @@ void nvUvmInterfaceMemoryCpuUnMap(uvmGpuAddressSpaceHandle vaSpace,
 EXPORT_SYMBOL(nvUvmInterfaceMemoryCpuUnMap);
 
 NV_STATUS nvUvmInterfaceChannelAllocate(uvmGpuAddressSpaceHandle  vaSpace,
-                                        uvmGpuChannelHandle *channel,
                                         const UvmGpuChannelAllocParams *allocParams,
-                                        UvmGpuChannelPointers * pointers)
+                                        uvmGpuChannelHandle *channel,
+                                        UvmGpuChannelInfo *channelInfo)
 {
     nvidia_stack_t *sp = NULL;
     NV_STATUS status;
@@ -514,9 +523,9 @@ NV_STATUS nvUvmInterfaceChannelAllocate(uvmGpuAddressSpaceHandle  vaSpace,
 
     status = rm_gpu_ops_channel_allocate(sp,
                                          (gpuAddressSpaceHandle)vaSpace,
-                                         (gpuChannelHandle *)channel,
                                          allocParams,
-                                         pointers);
+                                         (gpuChannelHandle *)channel,
+                                         channelInfo);
 
     nv_kmem_cache_free_stack(sp);
 
@@ -532,30 +541,6 @@ void nvUvmInterfaceChannelDestroy(uvmGpuChannelHandle channel)
 }
 EXPORT_SYMBOL(nvUvmInterfaceChannelDestroy);
 
-NV_STATUS nvUvmInterfaceCopyEngineAlloc(uvmGpuChannelHandle channel,
-                                        unsigned copyEngineIndex,
-                                        uvmGpuCopyEngineHandle *copyEngine,
-                                        UvmGpuChannelPointers *pointers)
-{
-    nvidia_stack_t *sp = NULL;
-    NV_STATUS status;
-
-    if (nv_kmem_cache_alloc_stack(&sp) != 0)
-    {
-        return NV_ERR_NO_MEMORY;
-    }
-
-    status = rm_gpu_ops_copy_engine_alloc(sp,
-                                          (gpuChannelHandle) channel,
-                                          copyEngineIndex,
-                                          (gpuObjectHandle *) copyEngine,
-                                          pointers);
-
-    nv_kmem_cache_free_stack(sp);
-    return status;
-}
-EXPORT_SYMBOL(nvUvmInterfaceCopyEngineAlloc);
-
 NV_STATUS nvUvmInterfaceQueryCaps(uvmGpuAddressSpaceHandle vaSpace,
                                   UvmGpuCaps * caps)
 {
@@ -988,10 +973,54 @@ void nvUvmInterfaceDeRegisterUvmOps(void)
     // Note that since we dropped the lock, another set of callbacks could have
     // already been registered. That's ok, since we just need to wait for old
     // ones to finish.
-    NV_ON_EACH_CPU(flush_top_half, NULL);
+    on_each_cpu(flush_top_half, NULL, 1);
 }
 EXPORT_SYMBOL(nvUvmInterfaceDeRegisterUvmOps);
 
+NV_STATUS nv_uvm_suspend(void)
+{
+    NV_STATUS status = NV_OK;
+    struct UvmOpsUvmEvents *events;
+
+    // Synchronize callbacks with unregistration
+    down(&g_pNvUvmEventsLock);
+
+    // It's not strictly necessary to use a cached local copy of the events
+    // pointer here since it can't change under the lock, but we'll do it for
+    // consistency.
+    events = getUvmEvents();
+    if (events && events->suspend)
+    {
+        status = events->suspend();
+    }
+
+    up(&g_pNvUvmEventsLock);
+
+    return status;
+}
+
+NV_STATUS nv_uvm_resume(void)
+{
+    NV_STATUS status = NV_OK;
+    struct UvmOpsUvmEvents *events;
+
+    // Synchronize callbacks with unregistration
+    down(&g_pNvUvmEventsLock);
+
+    // It's not strictly necessary to use a cached local copy of the events
+    // pointer here since it can't change under the lock, but we'll do it for
+    // consistency.
+    events = getUvmEvents();
+    if (events && events->resume)
+    {
+        status = events->resume();
+    }
+
+    up(&g_pNvUvmEventsLock);
+
+    return status;
+}
+
 void nv_uvm_notify_start_device(const NvU8 *pUuid)
 {
     NvProcessorUuid uvmUuid;
@@ -1257,4 +1286,16 @@ NV_STATUS nvUvmInterfaceReportNonReplayableFault(uvmGpuAddressSpaceHandle vaSpac
 }
 EXPORT_SYMBOL(nvUvmInterfaceReportNonReplayableFault);
 
+#else // NV_UVM_ENABLE
+
+NV_STATUS nv_uvm_suspend(void)
+{
+    return NV_OK;
+}
+
+NV_STATUS nv_uvm_resume(void)
+{
+    return NV_OK;
+}
+
 #endif // NV_UVM_ENABLE
